from metabotk.main import MetaboTK as mtk

a = mtk().io.from_excel(
    "tests/test_data/cdt_demo.xlsx", sample_id_column="PARENT_SAMPLE_NAME"
)
b = a.ops.subset(what="samples", ids=a.samples[0:10])


print(b.stats.sample_stats())



from typing import Literal
import numpy as np
import pandas as pd
from metabotk.utils import validate_dataframe

"""
Module containing functions to detect, count and remove outlier values
"""


def detect_outliers(data, threshold):
    """
    Detect outlier values in a single-column numerical array.

    Parameters:
    - data: single-column numerical array
    - threshold: a factor that determines the range from the IQR

    Returns:
    - Boolean array indicating outliers (True) and non-outliers (False)
    """
    if len(data) == 0:
        raise ValueError("Input is empty.")
    if isinstance(data, pd.DataFrame):
        raise TypeError("DataFrame input is not supported.")
    if np.isnan(data).all():
        return np.zeros(len(data), dtype=bool)
    median = np.nanmedian(data)
    q1 = np.nanquantile(data, 0.25)
    q3 = np.nanquantile(data, 0.75)
    iqr = q3 - q1
    cutoff_lower = median - (threshold * iqr)
    cutoff_upper = median + (threshold * iqr)
    is_outlier = (data < cutoff_lower) | (data > cutoff_upper)
    return is_outlier


def get_outliers_matrix(
    data_frame: pd.DataFrame, threshold: float, axis: Literal[0, 1] = 0
):
    """
    Get a matrix indicating outliers in each row or column of a dataframe.

    Parameters:
    - data_frame: pandas DataFrame containing only numeric values
    - threshold: a factor that determines the range from the IQR
    - axis: {0 or ‘index’, apply to each column, 1 or ‘columns’, apply to each row}, default 0

    Returns:
    - pandas DataFrame indicating outliers (True) and non-outliers (False)
    """
    validate_dataframe(data_frame)
    matrix = data_frame.apply(lambda x: detect_outliers(x, threshold), axis=axis)
    return matrix


def count_outliers(data_frame: pd.DataFrame, threshold: float, axis: Literal[0, 1] = 0):
    """
    Count number of outlier values in each row or column of a dataframe.

    Parameters:
    - data_frame: pandas DataFrame containing only numeric values
    - threshold: a factor that determines the range from the IQR
    - axis: {0 or ‘index’, apply to each column, 1 or ‘columns’, apply to each row}, default 0

    Returns:
    - pandas Series with the row/column index and the number of outliers
    """
    validate_dataframe(data_frame)
    outliers_matrix = get_outliers_matrix(data_frame, threshold)
    outlier_counts = outliers_matrix.sum(axis=axis)
    return outlier_counts


def remove_outliers(
    data_frame: pd.DataFrame, threshold: float, axis: Literal[0, 1] = 0
):
    """
    Replace outlier values with NAs in a dataframe, column-wise or row-wise.

    Parameters:
    - data_frame: pandas DataFrame containing only numeric values
    - threshold: a factor that determines the range from the IQR
    - axis: {0 or ‘index’, apply to each column, 1 or ‘columns’, apply to each row}, default 0

    Returns:
    - pandas DataFrame where the outlier values are replaced by NAs
    """
    validate_dataframe(data_frame)
    outliers = get_outliers_matrix(data_frame, threshold, axis=axis)
    data_frame_without_outliers = data_frame.where(~outliers, np.nan)
    return data_frame_without_outliers



import pandas as pd
import numpy as np


##Create a test dataframe with outlier values
def create_test_dataframe_with_outliers():
    np.random.seed(42)
    test_data = {
        "A": np.random.normal(
            loc=10, scale=2, size=5
        ),  # Normal distribution without outliers
        "B": np.random.normal(
            loc=20, scale=3, size=5
        ),  # Normal distribution without outliers
        "C": np.random.normal(
            loc=30, scale=4, size=5
        ),  # Normal distribution without outliers
        "D": np.random.normal(
            loc=40, scale=5, size=5
        ),  # Normal distribution without outliers
    }
    test_data["B"][0] = 100  # Outlier in column B
    test_data["D"][1] = -50  # Outlier in column D
    test_dataframe = pd.DataFrame(test_data)
    return test_dataframe


def create_outlier_presence_matrix_from_dataframe():
    test_dataframe = create_test_dataframe_with_outliers()
    outliers_df = pd.DataFrame(
        False, index=test_dataframe.index, columns=test_dataframe.columns
    )
    outliers_df.loc[0, "B"] = True  # Outlier in column B
    outliers_df.loc[1, "D"] = True  # Outlier in column B
    return outliers_df


##Create a test dataframe with missing values
def create_test_dataframe_with_missing():
    np.random.seed(42)
    test_data = {
        "A": np.random.normal(
            loc=10, scale=2, size=5
        ),  # Normal distribution without missings
        "B": np.random.normal(
            loc=20, scale=3, size=5
        ),  # Normal distribution without missings
        "C": np.random.normal(
            loc=30, scale=4, size=5
        ),  # Normal distribution without missings
        "D": np.random.normal(
            loc=40, scale=5, size=5
        ),  # Normal distribution without missings
    }
    test_data["B"][0] = np.nan  # missing in column B
    test_data["B"][1] = np.nan  # missing in column B
    test_data["B"][2] = np.nan  # missing in column B
    test_data["D"][1] = np.nan  # missing in column D
    test_data["A"][1] = np.nan  # missing in column D
    test_data["D"][1] = np.nan  # missing in column D

    test_dataframe = pd.DataFrame(test_data)
    return test_dataframe



import pandas as pd
from metabotk.parse_and_setup import (
    read_excel,
    read_prefix,
    dataset_from_prefix,
    read_tables,
)

"""
Setup dataset from file(s)
"""


class DatasetIO:
    def __init__(self, dataset):
        self.dataset = dataset

    def from_excel(
        self,
        file_path: str,
        sample_metadata_sheet: str = "Sample Meta Data",
        chemical_annotation_sheet: str = "Chemical Annotation",
        data_sheet: str = "Data",
        sample_id_column: str = "sample",
        metabolite_id_column: str = "CHEM_ID",
    ):
        parsed = read_excel(
            file_path, sample_metadata_sheet, chemical_annotation_sheet, data_sheet
        )
        return self.dataset._setup(
            data=parsed["data"],
            sample_metadata=parsed["sample_metadata"],
            chemical_annotation=parsed["chemical_annotation"],
            sample_id_column=sample_id_column,
            metabolite_id_column=metabolite_id_column,
        )

    def from_tables(
        self,
        sample_metadata,
        chemical_annotation,
        data,
        sample_id_column: str = "sample",
        metabolite_id_column: str = "CHEM_ID",
    ):
        parsed = read_tables(sample_metadata, chemical_annotation, data)
        return self.dataset._setup(
            data=parsed["data"],
            sample_metadata=parsed["sample_metadata"],
            chemical_annotation=parsed["chemical_annotation"],
            sample_id_column=sample_id_column,
            metabolite_id_column=metabolite_id_column,
        )

    def from_prefix(
        self,
        prefix: str,
        sample_id_column: str = "sample",
        metabolite_id_column: str = "CHEM_ID",
    ):
        """

        Args:
            prefix:
            sample_id_column:
            metabolite_id_column:

        Returns:

        """
        parsed = read_prefix(prefix)
        return self.dataset._setup(
            data=parsed["data"],
            sample_metadata=parsed["sample_metadata"],
            chemical_annotation=parsed["chemical_annotation"],
            sample_id_column=sample_id_column,
            metabolite_id_column=metabolite_id_column,
        )

    """
    Save dataset to file(s)
    """

    def save_prefix(self, prefix: str):
        """

        Args:
            prefix:
        """
        prefix_dict = dataset_from_prefix(prefix)
        self.dataset.data.to_csv(prefix_dict["data"], sep="\t", index=True)
        self.dataset.sample_metadata.to_csv(
            prefix_dict["sample_metadata"], sep="\t", index=True
        )
        self.dataset.chemical_annotation.to_csv(
            prefix_dict["chemical_annotation"], sep="\t", index=True
        )
        print(f"Saved to {prefix}")

    def save_excel(self, file_path, data_sheet="Data"):
        """
        Save the dataset to an Excel file.

        This function saves the dataset to an Excel file. The data, chemical
        annotation, and sample metadata are saved to separate sheets in the
        Excel file. The name of the sheet containing the data is specified by
        the `data_name` parameter.

        Args:
            file_path (str): Path to save the Excel file.
            data_name (str): Name of the sheet containing the data. Default is
                "data".

        """
        with pd.ExcelWriter(file_path) as writer:
            self.dataset.chemical_annotation.to_excel(
                writer, sheet_name="Chemical Annotation"
            )
            self.dataset.sample_metadata.to_excel(writer, sheet_name="Sample Meta Data")
            self.dataset.data.to_excel(writer, sheet_name=data_sheet)



import numpy as np

import pandas as pd
from metabotk.utils import validate_dataframe
from typing import Literal

"""
Module for detecting and counting missing values in data,
as well as removing columns/rows based on their missing data content.

Methods:
    detect_missing(data): Detects missing values in a collection.
    count_missing(data_frame, axis=0): Counts missing values in each row or column of a DataFrame.
    drop_columns_with_missing(data_frame): Removes columns with missing values above the threshold.
    drop_rows_with_missing(data_frame): Removes rows with missing values above the threshold.
"""


def _validate_threshold(threshold):
    """
    Validates the threshold attribute.
    """
    if not isinstance(threshold, (int, float)):
        raise TypeError("Threshold must be a numeric value")
    if not 0 <= threshold <= 1:
        raise ValueError("Threshold must be between 0 and 1")


def _detect_missing(data):
    """
    Detects missing values in a collection.

    Parameters:
        data (list, array, or Series): Collection containing data.

    Returns:
        Boolean array indicating missing (True) and non-missing data (False).
    """

    is_missing = np.isnan(data)
    return is_missing


def count_missing(data):
    """
    Counts missing values in a collection.

    Parameters:
        data (list, array, or Series): Collection containing data.

    Returns:
        Number of missing values in collection
    """

    missing = _detect_missing(data)
    n_missing = missing.sum()
    return n_missing


def count_missing_in_dataframe(data_frame, axis=0):
    """
    Counts missing values in each row or column of a DataFrame.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing only numeric values.
        axis (int, optional): Axis along which to count missing values.
            0 for columns, 1 for rows. Default is 0.

    Returns:
        Series: Pandas Series with the row/column index and the number of missing values.
    """
    validate_dataframe(data_frame)
    missing_values = data_frame.apply(_detect_missing, axis=axis)
    n_missing_values = missing_values.sum(axis=axis)
    return n_missing_values


def _drop_columns_with_missing(data_frame, threshold=0.25):
    """
    Removes columns with missing values above the threshold.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing only numeric values.

    Returns:
        DataFrame: DataFrame with columns with missingness higher than the threshold.
    """
    _validate_threshold(threshold)
    validate_dataframe(data_frame)
    missing = count_missing_in_dataframe(data_frame, axis=0)
    to_drop = missing[missing / len(data_frame) > threshold]
    to_drop = data_frame[list(to_drop.index)]
    return to_drop


def _drop_rows_with_missing(data_frame, threshold=0.25):
    """
    Removes rows with missing values above the threshold.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing only numeric values.

    Returns:
        DataFrame: DataFrame with rows with missingness higher than the threshold.
    """
    _validate_threshold(threshold)
    validate_dataframe(data_frame)
    missing = count_missing_in_dataframe(data_frame, axis=1)
    to_drop = missing[missing / len(data_frame.columns) > threshold]
    to_drop = data_frame.loc[to_drop.index]
    return to_drop


def drop_missing_from_dataframe(
    data_frame: pd.DataFrame,
    threshold: float = 0.25,
    axis: Literal[0, 1] = 0,
):
    """
    Remove columns or rows with missing values above the threshold.

    Parameters:
        - threshold: missingness over which to remove the row/column
        - axis: {0 or ‘index’, remove columns, 1 or ‘columns’, remove rows}, default 0

    Returns:
        DataFrame: DataFrame containing the rows/columns dropped.
    """
    if axis == 0:
        to_drop = _drop_columns_with_missing(data_frame=data_frame, threshold=threshold)
        print(f"Removed {len(to_drop.columns)} columns")
        remaining_data = data_frame.drop(columns=to_drop.columns)
        return remaining_data

    elif axis == 1:
        to_drop = _drop_rows_with_missing(data_frame=data_frame, threshold=threshold)
        print(f"Removed {len(to_drop.index)} rows")
        remaining_data = data_frame.drop(index=to_drop.index)
        return remaining_data



import pandas as pd
import pytest
import numpy as np
from src.metabotk.utils import (
    validate_dataframe,
    ensure_numeric_data,
    parse_input,
    validate_new_data,
    reset_index_if_not_none,
)


class TestValidateNewShapeIndex:
    def test_validate_new_shape_index(self):
        data = pd.read_csv("tests/test_data/data.csv")
        data = data.set_index("PARENT_SAMPLE_NAME")
        new_data = pd.DataFrame(np.nan, index=data.index, columns=data.columns)
        validate_new_data(data, new_data)

    def test_different_shape(self):
        data = pd.read_csv("tests/test_data/data.csv")
        data = data.set_index("PARENT_SAMPLE_NAME")
        new_data = data.iloc[0:10]
        with pytest.raises(ValueError):
            validate_new_data(data, new_data)

    def test_different_index(self):
        data = pd.read_csv("tests/test_data/data.csv")
        data = data.set_index("PARENT_SAMPLE_NAME")
        new_data = data.copy()
        new_data.index.name = "INVALID_INDEX"
        with pytest.raises(ValueError):
            validate_new_data(data, new_data)


class TestValidateDataFrame:
    def test_valid_dataframe(self):
        df = pd.DataFrame({"A": [1, 2, 3], "B": [4, 5, 6]})
        assert validate_dataframe(df) is None

    def test_invalid_dataframe(self):
        with pytest.raises(TypeError):
            validate_dataframe([])

    def test_validate_dataframe_valid(self):
        # Input is a pandas DataFrame
        data_frame = pd.DataFrame({"A": [1, 2, 3], "B": [4, 5, 6]})
        validate_dataframe(data_frame)
        # No error should be raised

    def test_validate_dataframe_invalid(self):
        # Input is not a pandas DataFrame
        data_frame = [1, 2, 3]
        with pytest.raises(TypeError):
            validate_dataframe(data_frame)


class TestEnsureNumericData:
    def test_valid_numeric_data(self):
        data = [1, 2, 3, 4]
        result = ensure_numeric_data(data)
        assert np.array_equal(result, np.array(data))

    def test_invalid_empty_data(self):
        with pytest.raises(ValueError):
            ensure_numeric_data([])

    def test_invalid_non_numeric_data(self):
        with pytest.raises(TypeError):
            ensure_numeric_data(["a", "b", "c"])

    def test_valid_mixed_numeric_data(self):
        data = [1, 2, 3, "4"]
        with pytest.raises(TypeError):
            ensure_numeric_data(data)

    def test_valid_series_input(self):
        series = pd.Series([1, 2, 3, 4])
        result = ensure_numeric_data(series)
        assert np.array_equal(result, np.array(series))

    def test_invalid_empty_series_input(self):
        series = pd.Series([])
        with pytest.raises(ValueError):
            ensure_numeric_data(series)

    def test_invalid_non_numeric_series_input(self):
        series = pd.Series(["a", "b", "c"])
        with pytest.raises(TypeError):
            ensure_numeric_data(series)


def test_reset_index_if_not_none():
    data = pd.read_csv("tests/test_data/sample_metadata.csv")
    data_after = reset_index_if_not_none(data)
    assert data.index.name is None
    data = data.set_index("PARENT_SAMPLE_NAME")
    assert data.index.name == "PARENT_SAMPLE_NAME"
    data_after = reset_index_if_not_none(data)
    assert data_after.index.name is None


class TestParseInput:
    def test_parse_input_pandas_dataframe(self):
        input_data = pd.DataFrame({"A": [1, 2, 3], "B": [4, 5, 6]})
        assert parse_input(input_data).equals(input_data)

    def test_parse_input_tsv_file(self):
        tsv_file_path = "tests/test_data/data.tsv"
        assert isinstance(parse_input(tsv_file_path), pd.DataFrame)

    def test_parse_input_csv_file(self):
        csv_file_path = "tests/test_data/data.csv"
        assert isinstance(parse_input(csv_file_path), pd.DataFrame)

    def test_parse_input_invalid_input(self):
        invalid_input = 123  # Not a DataFrame or a file path
        with pytest.raises(TypeError):
            parse_input(invalid_input)

    def test_parse_input_invalid_extension(self):
        invalid_input = "tests/test_data/test.invalid"  # Not a DataFrame or a file path
        with pytest.raises(TypeError):
            parse_input(invalid_input)

    def test_parse_input_custom_prefix_ending(self):
        data = "tests/test_data/test.data"
        samples = "tests/test_data/test.samples"
        metabolites = "tests/test_data/test.metabolites"
        for i in [data, samples, metabolites]:
            assert isinstance(parse_input(i), pd.DataFrame)



import pandas as pd
import pytest
from metabotk.providers_handler import MetabolonCDT
import os


# Fixture to create a sample MetabolonCDT instance
@pytest.fixture
def metabolon_instance():
    return MetabolonCDT()


# Test case to check if required sheets are missing
def test_read_metabolon_excel_missing_sheets(metabolon_instance):
    # Create a mock Excel file with missing sheets
    mock_excel_path = "tests/mock_missing_sheets.xlsx"
    pd.DataFrame().to_excel(mock_excel_path)  # Empty Excel file

    # Test that ValueError is raised if required sheets are missing
    with pytest.raises(ValueError):
        metabolon_instance.import_excel(mock_excel_path)
    os.remove(mock_excel_path)


# Test case to check if sheets are correctly assigned
def test_read_metabolon_excel(metabolon_instance):
    test_excel_path = "data/cdt_demo.xlsx"
    # Test that sheets are correctly assigned to attributes
    metabolon_instance.import_excel(test_excel_path)
    assert metabolon_instance.sample_metadata is not None
    assert metabolon_instance.chemical_annotation is not None
    assert metabolon_instance.peak_area_data is not None


# Test case to check reading flat tables
def test_read_metabolon_flat_tables(metabolon_instance):
    # Mock data frames for sample metadata and chemical annotation
    sample_metadata_df = pd.DataFrame({"ID": [1, 2, 3], "Name": ["A", "B", "C"]})
    chemical_annotation_df = pd.DataFrame(
        {"Compound": ["X", "Y", "Z"], "Mass": [100, 200, 300]}
    )

    # Test reading flat tables
    metabolon_instance.import_tables(sample_metadata_df, chemical_annotation_df)
    assert metabolon_instance.sample_metadata.shape == (3, 2)
    assert metabolon_instance.chemical_annotation.shape == (3, 2)



# import miceforest as mf
import numpy as np
import pandas as pd

# from sklearn.utils import check_random_state
from metabotk.interface import MetaboTK

import pytest

from metabotk.statistics_handler import get_top_n_correlations


class TestImputationHandler:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.metabotk = MetaboTK(
            data_provider="metabolon",
            sample_id_column="PARENT_SAMPLE_NAME",
            metabolite_id_column="CHEM_ID",
        )
        self.metabotk.import_excel(
            "data/cdt_demo.xlsx", data_sheet="batch_normalized_data"
        )
        self.metabotk.data = self.metabotk.drop_missing_from_dataframe()
        self.metabotk._update_chemical_annotation()

    def test_miceforest_returns_dict(self):
        imputed_data = self.metabotk.imputation.miceforest(
            n_correlated_metabolites=10,
            n_imputed_datasets=1,
            n_iterations=1,
            random_state=42,
        )
        assert isinstance(imputed_data, dict)

    def test_miceforest_keys_start_from_1(self):
        imputed_data = self.metabotk.imputation.miceforest(
            n_correlated_metabolites=10,
            n_imputed_datasets=2,
            n_iterations=1,
            random_state=42,
        )
        assert all(key >= 1 for key in imputed_data.keys())

    def test_miceforest_returns_correct_number_of_datasets(self):
        n_imputed_datasets = 3
        imputed_data = self.metabotk.imputation.miceforest(
            n_correlated_metabolites=10,
            n_imputed_datasets=n_imputed_datasets,
            n_iterations=2,
            random_state=42,
        )
        assert len(imputed_data) == n_imputed_datasets

    def test_miceforest_returns_correct_shape_data(self):
        imputed_data = self.metabotk.imputation.miceforest(
            n_correlated_metabolites=10,
            n_imputed_datasets=2,
            n_iterations=2,
            random_state=42,
        )
        for dataset in imputed_data.values():
            assert dataset.shape == self.metabotk.data.shape

    def test_no_nan_after_imputation(self):
        imputed_data = self.metabotk.imputation.miceforest(
            n_correlated_metabolites=10,
            n_imputed_datasets=2,
            n_iterations=2,
            random_state=42,
        )

        for dataset in imputed_data.values():
            assert dataset.isnull().values.any() == False



"""
Statistics module
"""

import pandas as pd
import numpy as np
from typing import Literal
from metabotk.utils import ensure_numeric_data

import metabotk.outliers_handler as outliers
import metabotk.missing_handler as missing


def compute_correlations(
    data_frame: pd.DataFrame, method: str = "pearson"
) -> pd.DataFrame:
    """
    Computes correlations between columns in a pandas DataFrame.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing numerical values.
        method (str): Method for computing correlations. Default is 'pearson'.

    Returns:
        DataFrame: Pandas DataFrame containing correlations between columns.
    """
    correlations = data_frame.corr(method=method)
    return correlations


def get_top_n_correlations(
    data_frame: pd.DataFrame, n: int = 10, method: str = "pearson"
):
    """
    Get the top n correlations for each column in a pandas DataFrame.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing numerical values.
        n (int): Number of top correlations to return. Default is 10.
        method (str): Method for computing correlations. Default is 'pearson'.

    Returns:
        DataFrame: Pandas DataFrame containing top n correlations
        for each id and their values.
    """
    correlations = compute_correlations(data_frame, method)
    top_correlations = []

    for id in correlations.columns:
        top_n_abs = (
            correlations[id].abs().sort_values(ascending=False).drop(id).iloc[0:n].index
        )
        top_n = correlations[id].loc[top_n_abs]
        top_n.index.name = "id_2"
        top_n.name = "correlation"
        top_n = top_n.reset_index()
        top_n["id_1"] = id
        top_correlations.append(top_n)
    top_correlations = pd.concat(top_correlations)
    top_correlations = top_correlations[["id_1", "id_2", "correlation"]]
    return top_correlations


def coefficient_of_variation(data):
    """
    Compute the coefficient of variation in percentage.

    The coefficient of variation (CV) is a measure of the dispersion of a dataset.
    It is calculated as the standard deviation (σ) divided by the mean (μ).
    The CV is expressed as a percentage (%).

    Parameters:
        data (list, array, or Series): Collection containing numerical data.

    Returns:
        float: Coefficient of Variation % (CV%).
    """
    data = ensure_numeric_data(data)
    # Remove any missing (nan) values.
    data = data[~np.isnan(data)]
    # Compute the standard deviation.
    std = np.std(data)
    # Compute the mean.
    mean = np.mean(data)
    # Compute the coefficient of variation.
    cv = std / mean
    # Convert coefficient of variation to a percentage.
    cv_pctg = cv * 100
    return cv_pctg


def total_sum_abundance(data_frame, exclude_incomplete=True):
    """
    Computes total sum abundance (TSA) row-wise

    Parameters:
        data_frame (DataFrame): pandas DataFrame containing numerical data.
        exclude_incomplete (bool): option to exclude columns containing incomplete values
        from the computation

    Returns:
        Series: Pandas Series containing TSA values for each row.
    """
    if exclude_incomplete:
        data_frame = data_frame.dropna(axis=1).copy()
    else:
        data_frame = data_frame.copy()
    tsa = data_frame.sum(axis=1, skipna=True)
    tsa.name = "TSA"
    return tsa


def compute_statistics(data, outlier_threshold):
    """
    Computes basic statistics for a collection of numerical data.

    Parameters:
        data (list, array, or Series): Collection containing numerical data.
        outlier_threshold (float): Threshold for outlier detection.

    Returns:
        Series: Pandas Series containing basic statistics.

    Notes:
        The statistics computed are:
        - Mean
        - Standard deviation
        - Median
        - Min
        - Max
        - Sum
        - Coefficient of Variation (CV%)
        - Number of missing values
        - Number of outliers
    """
    data_series = pd.Series(data)
    if len(data) == 0:
        raise ValueError("Input data is empty")
    stats = data_series.describe()
    cv = coefficient_of_variation(data)
    stats["CV%"] = cv
    stats = stats.rename(index={"50%": "median"})
    stats["missing"] = missing.count_missing(data_series)
    stats["outliers"] = sum(
        outliers.detect_outliers(data_series, threshold=outlier_threshold)
    )
    return stats


def compute_dataframe_statistics(data_frame, outlier_threshold, axis):
    """
    Computes basic statistics for a pandas DataFrame.

    Parameters:
        data_frame (DataFrame): Pandas DataFrame containing numerical values.
        outlier_threshold (float): Threshold for outlier detection. Default is None.
        axis (int): Which axis to compute statistics on. Default is 0 (column-wise)

    Returns:
        DataFrame: Pandas DataFrame containing statistics for each column.

    Notes:
        The statistics computed are:
        - Mean
        - Standard deviation
        - Median
        - Min
        - Max
        - Sum
        - Coefficient of Variation (CV%)
        - Number of missing values
        - Number of outliers
    """
    stats = data_frame.apply(
        lambda x: compute_statistics(x, outlier_threshold=outlier_threshold),
        axis=axis,
    )
    if axis == 0:
        stats = stats.transpose()
    return stats


class Statistics:
    """
    Class for obtaining basic statistics about the data.

    This class provides methods for computing basic statistics
    (mean, standard deviation, median, min, max, sum, CV%, missing, outliers)
    for a collection of numerical data or a pandas DataFrame.
    """

    def __init__(self, dataset):
        """
        Initializes the StatisticsHandler.
        """

        self.dataset = dataset

    def show_methods(self):
        methods = [attr for attr in dir(self) if callable(getattr(self, attr))]

        user_methods = [method for method in methods if not method.startswith("__")]

        print(f"Available methods in {self.__class__.__name__}:")
        for method in user_methods:
            print(f"- {method}")

    def metabolite_stats(self, outlier_threshold=5):
        """
        Computes basic statistics for the metabolomics data metabolite-wise

        This function computes basic statistics for each metabolite in the data.
        The statistics computed are:
        - Mean
        - Standard deviation
        - Median
        - Min
        - Max
        - Sum
        - Coefficient of Variation (CV%)
        - Number of missing values
        - Number of outliers

        Parameters:
        - outlier_threshold: Threshold for outlier detection (default=5).

        Returns:
            pandas DataFrame: DataFrame containing statistics for each metabolite.
            The index of the DataFrame is the metabolite names.

        """
        # Ensure that data is set up properly
        if self.dataset.data.empty:
            raise ValueError(
                "No data available. Please import data before computing statistics."
            )

        # Compute statistics using StatisticsHandler
        metabolite_stats = compute_dataframe_statistics(
            self.dataset.data, outlier_threshold, axis=0
        )
        # self.metabolite_stats=metabolite_stats
        return metabolite_stats

    def sample_stats(self, outlier_threshold=5):
        """
        Computes basic statistics for the metabolomics data sample-wise

        This function computes statistics for each sample across all metabolites.

        Parameters:
            outlier_threshold (int): Threshold for identifying outliers.

        Returns:
             DataFrame containing statistics for each sample across all metabolites.
        """
        # Ensure that data is set up properly
        if self.dataset.data.empty:
            raise ValueError(
                "No data available. Please import data before computing statistics."
            )
        sample_stats = compute_dataframe_statistics(
            self.dataset.data, outlier_threshold, axis=1
        )

        # Compute Total Sum of Abundance (TSA) for each sample across all metabolites
        tsa_complete_only = total_sum_abundance(
            self.dataset.data, exclude_incomplete=True
        )
        tsa_complete_only.name = "TSA_complete_only"
        tsa_all = total_sum_abundance(self.dataset.data, exclude_incomplete=False)
        tsa_all.name = "TSA_including_incomplete"
        tsa = pd.concat([tsa_complete_only, tsa_all], axis=1)

        # Merge TSA with sample statistics
        sample_stats = sample_stats.merge(tsa, left_index=True, right_index=True)
        # self.sample_stats=sample_stats
        return sample_stats

    def corr(self, method: str = "pearson"):
        return compute_correlations(self.dataset.data, method)

    def top_corr(self, n: int = 10, method: str = "pearson"):
        return get_top_n_correlations(self.dataset.data, n, method)

    def remove_outliers(
        self, threshold: float, on: Literal["samples", "metabolites"] = "metabolites"
    ):
        if on == "metabolites":
            axis = 0
        elif on == "samples":
            axis = 1
        return outliers.remove_outliers(self.dataset.data, threshold, axis)

    def remove_missing(
        self, threshold: float, on: Literal["samples", "metabolites"] = "metabolites"
    ):
        if on == "metabolites":
            axis = 0
        elif on == "samples":
            axis = 1
        return missing.drop_missing_from_dataframe(self.dataset.data, threshold, axis)



import pandas as pd
from metabotk.parse_and_setup import (
    setup_data,
    setup_sample_metadata,
    setup_chemical_annotation,
)

from metabotk.utils import validate_new_data, validate_new_metadata


class MetabolomicDataset:
    """

    Attributes:
        sample_id_column:
        metabolite_id_column:
        data:
        sample_metadata:
        chemical_annotation:
        samples:
        metabolites:
    """

    def __init__(
        self,
        data: pd.DataFrame,
        sample_metadata: pd.DataFrame,
        chemical_annotation: pd.DataFrame,
        sample_id_column: str,
        metabolite_id_column: str,
    ) -> None:
        """
        Initialize the class.

        Parameters:
            sample_id_column (str): name of the sample id column
            metabolite_id_column (str): name of the metabolite id column
        """
        self._sample_id_column: str = sample_id_column
        self._metabolite_id_column: str = metabolite_id_column
        self.__data = data
        self.__sample_metadata = sample_metadata
        self.__chemical_annotation = chemical_annotation
        self.__samples = list(sample_metadata.index)
        self.__metabolites = list(chemical_annotation.index)

    @classmethod
    def _setup(
        cls,
        data: pd.DataFrame,
        sample_metadata: pd.DataFrame,
        chemical_annotation: pd.DataFrame,
        sample_id_column: str,
        metabolite_id_column: str,
    ):
        data = setup_data(data, sample_id_column)
        sample_metadata = setup_sample_metadata(sample_metadata, sample_id_column)
        chemical_annotation = setup_chemical_annotation(
            chemical_annotation, metabolite_id_column
        )
        chemical_annotation.index = chemical_annotation.index.map(str)
        sample_metadata.index = sample_metadata.index.map(str)
        data.columns = data.columns.map(str)
        data.index = data.index.map(str)
        metabolites = [i for i in data.columns if i in list(chemical_annotation.index)]
        data = data[metabolites]
        sample_metadata = sample_metadata.loc[data.index]
        sample_metadata.index.name = sample_id_column
        chemical_annotation = chemical_annotation.loc[data.columns]
        chemical_annotation.index.name = metabolite_id_column

        return cls(
            data=data,
            sample_metadata=sample_metadata,
            chemical_annotation=chemical_annotation,
            sample_id_column=sample_id_column,
            metabolite_id_column=metabolite_id_column,
        )

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, new_data):
        validate_new_data(self.data, new_data)
        self.__data = new_data

    @property
    def sample_metadata(self):
        return self.__sample_metadata

    @sample_metadata.setter
    def sample_metadata(self, new_sample_metadata):
        validate_new_metadata(self.sample_metadata, new_sample_metadata)
        self.__sample_metadata = new_sample_metadata

    @property
    def chemical_annotation(self):
        return self.__chemical_annotation

    @chemical_annotation.setter
    def chemical_annotation(self, new_chemical_annotation):
        validate_new_metadata(self.chemical_annotation, new_chemical_annotation)
        self.__chemical_annotation = new_chemical_annotation

    @property
    def samples(self) -> list[str]:
        return self.__samples

    @samples.setter
    def samples(self, new_samples):
        if len(new_samples) != len(self.data):
            raise ValueError("Number of samples must match number of data rows")
        self.__samples = new_samples

    @property
    def metabolites(self) -> list[str]:
        return list(self.chemical_annotation.index)

    @metabolites.setter
    def metabolites(self, new_metabolites):
        if len(new_metabolites) != len(self.data.columns):
            raise ValueError("Number of metabolites must match number of data columns")
        self.__metabolites = new_metabolites



import pytest
from metabotk.metabolomic_dataset import MetabolomicDataset
import pandas as pd
import numpy as np


@pytest.fixture
def dataset():
    dataset = MetabolomicDataset._setup(
        data=pd.read_csv("tests/test_data/data.csv"),
        sample_metadata=pd.read_csv("tests/test_data/sample_metadata.csv"),
        chemical_annotation=pd.read_csv("tests/test_data/chemical_annotation.csv"),
        sample_id_column="PARENT_SAMPLE_NAME",
        metabolite_id_column="CHEM_ID",
    )
    return dataset


class TestMetabolomicDataset:
    def test_setup(self, dataset):
        assert dataset._sample_id_column == "PARENT_SAMPLE_NAME"
        assert dataset._metabolite_id_column == "CHEM_ID"
        pd.testing.assert_index_equal(dataset.sample_metadata.index, dataset.data.index)
        assert list(dataset.data.columns) == list(dataset.chemical_annotation.index)

    def test_data_setter(self, dataset):
        new_data = pd.DataFrame(
            np.nan, index=dataset.data.index, columns=dataset.data.columns
        )
        dataset.data = new_data

    def test_sample_metadata_setter(self, dataset):
        new_sample_metadata = pd.DataFrame(
            np.nan,
            index=dataset.sample_metadata.index,
            columns=dataset.sample_metadata.columns,
        )
        dataset.sample_metadata = new_sample_metadata

    def test_chemical_annotation_setter(self, dataset):
        new_chemical_annotation = pd.DataFrame(
            np.nan,
            index=dataset.chemical_annotation.index,
            columns=dataset.chemical_annotation.columns,
        )
        dataset.chemical_annotation = new_chemical_annotation

    def test_samples(self, dataset):
        assert dataset.samples == list(dataset.sample_metadata.index)

    def test_samples_setter(self, dataset):
        new_samples = [np.nan for i in dataset.samples]
        dataset.samples = new_samples

    def test_samples_setter_diff_len(self, dataset):
        new_samples = [np.nan for i in dataset.samples[0:10]]
        with pytest.raises(ValueError):
            dataset.samples = new_samples

    def test_metabolites_setter(self, dataset):
        new_metabolites = [np.nan for i in dataset.metabolites]
        dataset.metabolites = new_metabolites

    def test_metabolites_setter_diff_len(self, dataset):
        new_metabolites = [np.nan for i in dataset.metabolites[0:10]]
        with pytest.raises(ValueError):
            dataset.metabolites = new_metabolites



from doctest import script_from_examples
import pytest
import pandas as pd
from metabotk.metabolomic_dataset import MetabolomicDataset
from metabotk.dataset_operations import DatasetOperations


@pytest.fixture
def ops():
    dataset = MetabolomicDataset._setup(
        data=pd.read_csv("tests/test_data/data.csv"),
        sample_metadata=pd.read_csv("tests/test_data/sample_metadata.csv"),
        chemical_annotation=pd.read_csv("tests/test_data/chemical_annotation.csv"),
        sample_id_column="PARENT_SAMPLE_NAME",
        metabolite_id_column="CHEM_ID",
    )
    ops = DatasetOperations(dataset)
    return ops


class TestOperations:
    def test_subset_samples(self, ops):
        samples = ["INTR-03208 [COPY 2]", "INTR-03200 [COPY 2]"]
        subsetted = ops.subset(what="samples", ids=samples)
        assert subsetted.chemical_annotation.equals(ops.dataset.chemical_annotation)
        assert subsetted.data.equals(ops.dataset.data.loc[samples])
        assert subsetted.sample_metadata.equals(
            ops.dataset.sample_metadata.loc[samples]
        )
        assert len(subsetted.data) == 2
        assert len(subsetted.sample_metadata) == 2
        assert len(subsetted.samples) == 2
        assert subsetted.samples == samples
        assert len(subsetted.metabolites) == len(ops.dataset.metabolites)

    def test_subset_metabolites(self, ops):
        metabolites = ["50", "100008998"]
        subsetted = ops.subset(what="metabolites", ids=metabolites)
        assert subsetted.chemical_annotation.equals(
            ops.dataset.chemical_annotation.loc[metabolites]
        )
        assert subsetted.data.equals(ops.dataset.data[metabolites])
        assert subsetted.chemical_annotation.equals(
            ops.dataset.chemical_annotation.loc[metabolites]
        )
        assert len(subsetted.data) == len(ops.dataset.data)
        assert len(subsetted.chemical_annotation) == 2
        assert len(subsetted.samples) == len(ops.dataset.samples)
        assert subsetted.metabolites == metabolites
        assert len(subsetted.metabolites) == len(metabolites)

    def test_replace_sample_names_in_data(self, ops):
        renamed = ops.replace_sample_names_in_data(new_index="CLIENT_IDENTIFIER")
        assert (
            renamed.samples
            == ops.dataset.sample_metadata["CLIENT_IDENTIFIER"].astype(str).tolist()
        )
        assert list(renamed.sample_metadata.index) == renamed.samples
        assert list(renamed.data.index) == renamed.samples

    def test_replace_sample_names_in_data_wrong_colname(self, ops):
        with pytest.raises(ValueError):
            ops.replace_sample_names_in_data(new_index="INVALID_NAME")



from metabotk.metabolomic_dataset import MetabolomicDataset
from metabotk.dataset_io import DatasetIO
from metabotk.dataset_operations import DatasetOperations
from metabotk.statistics_handler import Statistics
from metabotk.feature_selection import FeatureSelection
from metabotk.models_handler import ModelsHandler
from metabotk.dimensionality_reduction import DimensionalityReduction
from metabotk.visualization_handler import Visualization
import pandas as pd


def set_pandas_display_options() -> None:
    """Set pandas display options."""
    # Ref: https://stackoverflow.com/a/52432757/
    display = pd.options.display
    display.max_columns = 1000
    display.max_rows = 10_000
    display.max_colwidth = 199
    display.width = 1000
    # display.precision = 2  # set as needed
    # display.float_format = lambda x: '{:,.2f}'.format(x)  # set as needed


set_pandas_display_options()


class MetaboTK(MetabolomicDataset):
    def __init__(
        self,
        data=pd.DataFrame(),
        sample_metadata=pd.DataFrame(),
        chemical_annotation=pd.DataFrame(),
        sample_id_column="sample",
        metabolite_id_column="CHEM_ID",
    ) -> None:
        super().__init__(
            data=data,
            sample_metadata=sample_metadata,
            chemical_annotation=chemical_annotation,
            sample_id_column=sample_id_column,
            metabolite_id_column=metabolite_id_column,
        )

    @property
    def io(self):
        """Lazy initialization of DatasetIO instance."""
        if not hasattr(self, "_io_"):
            self._io_ = DatasetIO(self)
        return self._io_

    @property
    def ops(self):
        """Lazy initialization of DatasetEditor instance."""
        if not hasattr(self, "_operations_"):
            self._operations_ = DatasetOperations(self)
        return self._operations_

    @property
    def stats(self):
        """Lazy initialization of Statistics instance."""
        if not hasattr(self, "_statistics_"):
            self._statistics_ = Statistics(self)
        return self._statistics_

    @property
    def fs(self):
        """Lazy initialization of FeatureSelection instance."""
        if not hasattr(self, "_feature_selection_"):
            self._feature_selection_ = FeatureSelection(self)
        return self._feature_selection_

    @property
    def lm(self):
        """Lazy initialization of ModelsHandler instance."""
        if not hasattr(self, "_models_handler_"):
            self._models_handler_ = ModelsHandler(self)
        return self._models_handler_

    @property
    def dimred(self):
        """Lazy initialization of DimensionalityReduction instance."""
        if not hasattr(self, "_dimensionality_reduction_"):
            self._dimensionality_reduction_ = DimensionalityReduction(self)
        return self._dimensionality_reduction_

    @property
    def viz(self):
        """Lazy initialization of VisualizationHandler instance."""
        if not hasattr(self, "_visualization_"):
            self._visualization_ = Visualization(self)
        return self._visualization_



class ScalingHandler:
    """
    Class for performing scaling procedures on metabolomics data.
    """

    def __init__(
        self,
        dataset_manager,
    ):
        """
        Initialize the class.
        """
        self._dataset_manager = dataset_manager

    def tsa(self, inplace=True):
        """
        Scale by dividing each metabolite value by the Total Sum Abundance (TSA) of its sample
        """
        tsa = self._dataset_manager.data.sum(axis=1).to_dict()
        scaled = self._dataset_manager.data.apply(lambda x: x / tsa[x.name], axis=1)
        if inplace:
            self._dataset_manager.data = scaled
        else:
            return scaled

    def median(self, inplace=True):
        """
        Scale by dividing each metabolite value by the median of the metabolite values across all samples
        """
        scaled = self._dataset_manager.data / self._dataset_manager.data.median()
        if inplace:
            self._dataset_manager.data = scaled
        else:
            return scaled



import pytest
import metabotk.outliers_handler as outliers
import numpy as np
import pandas as pd
from tests.testing_functions import (
    create_test_dataframe_with_outliers,
    create_outlier_presence_matrix_from_dataframe,
)

# detect_outliers

test_data = pd.read_csv("tests/test_data/data.csv")
test_data = test_data[test_data.columns[5:10]].iloc[10:20]
test_values_series = test_data["212"]


class TestDetectOutliers:
    data = test_data

    def test_input_df(self):
        with pytest.raises(TypeError):
            outliers.detect_outliers(self.data, threshold=1)

    def test_input_series(self):
        assert (outliers.detect_outliers(test_values_series, threshold=1.5)).sum() == 1

    def test_input_list(self):
        data = [1, 2, 3, 4, 100]
        assert any(outliers.detect_outliers(data, threshold=1))

    def test_input_series_single_outlier(self):
        data = pd.Series([1, 2, 3, 4, 100])
        assert any(outliers.detect_outliers(data, threshold=1))

    def test_no_outliers(self):
        data = np.array([1, 2, 3, 4, 5])
        assert not any(outliers.detect_outliers(data, threshold=1))

    def test_single_outlier(self):
        data = np.array([1, 2, 3, 10, 5])
        assert np.array_equal(
            outliers.detect_outliers(data, threshold=1),
            [False, False, False, True, False],
        )

    def test_single_outlier_higher_threshold(self):
        data = np.array([1, 2, 3, 10, 5])
        assert np.array_equal(
            outliers.detect_outliers(data, threshold=5),
            [False, False, False, False, False],
        )

    def test_multiple_outliers(self):
        data = np.array([1, 100, 1, 50, 1, 2, 3])
        assert np.array_equal(
            outliers.detect_outliers(data, threshold=1),
            [False, True, False, True, False, False, False],
        )

    def test_negative_values(self):
        data = np.array([-1000, -5, 0, 5, 10])
        assert any(outliers.detect_outliers(data, threshold=1))

    def test_empty_input(self):
        with pytest.raises(ValueError):
            outliers.detect_outliers([], threshold=1)


# get_outliers_matrix


class TestGetOutliersMatrix:
    data, outliers_df = (
        create_test_dataframe_with_outliers(),
        create_outlier_presence_matrix_from_dataframe(),
    )

    def test_outliers_matrix(self):
        assert outliers.get_outliers_matrix(self.data, threshold=5).equals(
            self.outliers_df
        )


# count_outliers


class TestCountOutliers:
    data, outliers_df = (
        create_test_dataframe_with_outliers(),
        create_outlier_presence_matrix_from_dataframe(),
    )

    def test_count_outliers_column_wise(self):
        outlier_counts = outliers.count_outliers(self.data, axis=0, threshold=1)
        outlier_counts_to_assert = pd.Series([1, 2, 1, 2])
        outlier_counts_to_assert.index = self.data.columns
        assert outlier_counts_to_assert.equals(outlier_counts)

    def test_count_outliers_row_wise(self):
        outlier_counts = outliers.count_outliers(self.data, axis=1, threshold=1)
        outlier_counts_to_assert = pd.Series([1, 1, 1, 3, 0])
        outlier_counts_to_assert.index = self.data.index
        assert outlier_counts_to_assert.equals(outlier_counts)

    def test_count_outliers_column_wise_high_threshold(self):
        outlier_counts = outliers.count_outliers(self.data, axis=0, threshold=5)
        outlier_counts_to_assert = pd.Series([0, 1, 0, 1])
        outlier_counts_to_assert.index = self.data.columns
        assert outlier_counts_to_assert.equals(outlier_counts)

    def test_count_outliers_row_wise_high_threshold(self):
        outlier_counts = outliers.count_outliers(self.data, axis=1, threshold=5)
        outlier_counts_to_assert = pd.Series([1, 1, 0, 0, 0])
        outlier_counts_to_assert.index = self.data.index
        assert outlier_counts_to_assert.equals(outlier_counts)

    def test_count_outliers_column_wise_medium_threshold(self):
        outlier_counts = outliers.count_outliers(self.data, axis=0, threshold=1.2)
        outlier_counts_to_assert = pd.Series([1, 1, 0, 2])
        outlier_counts_to_assert.index = self.data.columns
        assert outlier_counts_to_assert.equals(outlier_counts)

    def test_count_outliers_row_wise_medium_threshold(self):
        outlier_counts = outliers.count_outliers(self.data, axis=1, threshold=1.2)
        outlier_counts_to_assert = pd.Series([1, 1, 1, 1, 0])
        outlier_counts_to_assert.index = self.data.index
        assert outlier_counts_to_assert.equals(outlier_counts)


# remove_outliers


class TestRemoveOutliers:
    data, outliers_df = (
        create_test_dataframe_with_outliers(),
        create_outlier_presence_matrix_from_dataframe(),
    )

    def test_outlier_removal(self):
        data_without_outliers = self.data.copy()
        data_without_outliers.loc[0, "B"] = np.nan  # Outlier in column B
        data_without_outliers.loc[1, "D"] = np.nan  # Outlier in column D
        assert outliers.remove_outliers(self.data, threshold=5).equals(
            data_without_outliers
        )



import pandas as pd
import os
import warnings
from metabotk.utils import parse_input, reset_index_if_not_none


def read_excel(
    file_path: str | os.PathLike[str],
    sample_metadata_sheet: str = "Sample Meta Data",
    chemical_annotation_sheet: str = "Chemical Annotation",
    data_sheet: str = "Data",
) -> dict[str, pd.DataFrame]:
    sheets = pd.read_excel(file_path, sheet_name=None)
    dataset_dict = {
        "sample_metadata": sheets.pop(sample_metadata_sheet),
        "chemical_annotation": sheets.pop(chemical_annotation_sheet),
        "data": sheets.pop(data_sheet),
    }
    return dataset_dict


def read_tables(
    sample_metadata: str | os.PathLike[str] | pd.DataFrame,
    chemical_annotation: str | os.PathLike[str] | pd.DataFrame,
    data: str | os.PathLike[str] | pd.DataFrame,
) -> dict[str, pd.DataFrame]:
    """

    Args:
        sample_metadata:
        chemical_annotation:
        data:

    Returns:

    """
    dataset_dict = {
        "sample_metadata": parse_input(sample_metadata),
        "chemical_annotation": parse_input(chemical_annotation),
        "data": parse_input(data),
    }
    return dataset_dict


def dataset_from_prefix(prefix: str) -> dict[str, str]:
    """

    Args:
        prefix:
    Returns:

    """
    prefix_dict = {
        "sample_metadata": f"{prefix}.samples",
        "chemical_annotation": f"{prefix}.metabolites",
        "data": f"{prefix}.data",
    }
    return prefix_dict


def read_prefix(prefix: str) -> dict[str, pd.DataFrame]:
    """
    Parse files from prefix
    Args:
        prefix: prefix valid for all three dataset files
    Returns:
        Dict of dataframes
    """
    prefix_dict = dataset_from_prefix(prefix)
    return read_tables(
        sample_metadata=prefix_dict["sample_metadata"],
        chemical_annotation=prefix_dict["chemical_annotation"],
        data=prefix_dict["data"],
    )


"""
Functions to setup dataset files for the main class 
"""


def setup_data(data: pd.DataFrame, sample_id_column: str):
    """

    Args:
        data:
        sample_id_column:

    Returns:

    """
    data = reset_index_if_not_none(data)
    if sample_id_column not in data.columns:
        raise ValueError(f"No sample ID column '{sample_id_column}' found in data")
    data.columns = [str(i) for i in data.columns]
    data.set_index(sample_id_column, inplace=True)
    return data


def setup_sample_metadata(sample_metadata: pd.DataFrame, sample_id_column: str):
    """
    Args:
        sample_metadata:
        sample_id_column:
        data:

    Returns:


    Raises:
        ValueError:
    """

    sample_metadata = reset_index_if_not_none(sample_metadata)
    # check that sample ID column is found in data
    if sample_id_column in sample_metadata.columns:
        # set metadata and data
        if sample_metadata[sample_id_column].duplicated().any():
            warnings.warn(
                "Warning: there are duplicate values in the chosen sample column.\
                        Consider choosing another column or renaming the duplicated samples"
            )
        # sample_metadata[sample_id_column] = sample_metadata[sample_id_column].astype(
        #    str
        # )
        sample_metadata.set_index(sample_id_column, inplace=True)
    else:
        raise ValueError(f"No sample ID column '{sample_id_column}' found in data")
    return sample_metadata


def setup_chemical_annotation(
    chemical_annotation: pd.DataFrame, metabolite_id_column: str
):
    """

    Args:
        chemical_annotation:
        metabolite_id_column:

    Returns:


    Raises:
        ValueError:
    """

    chemical_annotation = reset_index_if_not_none(chemical_annotation)
    # check that metabolite ID column is found in chemical annotation
    if metabolite_id_column in list(chemical_annotation.columns):
        if chemical_annotation[metabolite_id_column].duplicated().any():
            warnings.warn(
                "Warning: there are duplicate values in the chosen metabolite column.\
                        Consider choosing another column or renaming the duplicated metabolites"
            )

        chemical_annotation[metabolite_id_column] = chemical_annotation[
            metabolite_id_column
        ]  # .astype(str)
        chemical_annotation.set_index(metabolite_id_column, inplace=True)
    else:
        raise ValueError("No metabolite ID column found in chemical annotation")
    return chemical_annotation



import pandas as pd
from typing import Literal


class DatasetOperations:
    """
    Operation functions (subset, drop, sort, split)
    """

    def __init__(self, dataset):
        self.dataset = dataset

    def subset(
        self,
        what: Literal["samples", "metabolites"] = "samples",
        ids: list[str] | str = [],
    ):
        """

        Args:
            what:
            ids:

        Returns:

        """
        if what == "samples":
            return self._subset_samples(ids)
        elif what == "metabolites":
            return self._subset_metabolites(ids)

    def _subset_samples(self, samples_to_subset: list[str] | str):
        if not isinstance(samples_to_subset, list):
            samples_to_subset = list(samples_to_subset)
        remaining_data = self.dataset.data.loc[samples_to_subset]
        remaining_sample_metadata = self.dataset.sample_metadata.loc[samples_to_subset]
        return self.dataset._setup(
            data=remaining_data,
            sample_metadata=remaining_sample_metadata,
            chemical_annotation=self.dataset.chemical_annotation,
            sample_id_column=self.dataset._sample_id_column,
            metabolite_id_column=self.dataset._metabolite_id_column,
        )

    def _subset_metabolites(self, metabolites_to_subset: list[str] | str):
        """

        Args:
            metabolites_to_subset:

        Returns:

        """
        if not isinstance(metabolites_to_subset, list):
            metabolites_to_subset = list(metabolites_to_subset)

        remaining_data = self.dataset.data[metabolites_to_subset]

        remaining_chemical_annotation = self.dataset.chemical_annotation.loc[
            metabolites_to_subset
        ]
        return self.dataset._setup(
            data=remaining_data,
            sample_metadata=self.dataset.sample_metadata,
            chemical_annotation=remaining_chemical_annotation,
            sample_id_column=self.dataset._sample_id_column,
            metabolite_id_column=self.dataset._metabolite_id_column,
        )

    def drop(
        self,
        what: Literal["samples", "metabolites"] = "samples",
        ids: list[str] | str = [],
    ):
        """

        Args:
            what:
            ids:

        Returns:

        """
        if what == "samples":
            return self._drop_samples(ids)
        elif what == "metabolites":
            return self._drop_metabolites(ids)

    def _drop_samples(self, samples_to_drop: list[str] | str):
        """
        Drop specified samples from the dataset.
        Args:
            samples_to_drop:

        Returns:

        """
        if not isinstance(samples_to_drop, list):
            samples_to_drop = list(samples_to_drop)
        remaining_samples = list(
            set(self.dataset.samples).difference(set(samples_to_drop))
        )
        return self._subset_samples(remaining_samples)

    def _drop_metabolites(self, metabolites_to_drop: list[str] | str):
        """
        Drop specified metabolites from the dataset.
        Args:
            metabolites_to_drop:

        Returns:

        """
        remaining_metabolites = list(
            set(self.dataset.metabolites).difference(metabolites_to_drop)
        )
        return self._subset_metabolites(remaining_metabolites)

    def sort(
        self,
        on: Literal["samples", "metabolites"] = "samples",
        by: list[str] = [],
        ascending: bool = True,
    ):
        """

        Args:
            on:
            by:
            ascending:

        Returns:

        """
        if on == "samples":
            return self._sort_samples(by, ascending)
        elif on == "metabolites":
            return self._sort_metabolites(by, ascending)

    def _sort_samples(self, by: list[str] | str, ascending: bool = True):
        """

        Args:
            by:
            ascending:

        Returns:

        """
        sorted_ids = self.dataset.sample_metadata.sort_values(
            by=by, ascending=ascending
        ).index
        return self.subset(what="samples", ids=list(sorted_ids))

    def _sort_metabolites(self, by: list[str] | str, ascending: bool = True):
        """

        Args:
            by:
            ascending:

        Returns:

        """
        sorted_ids = self.dataset.chemical_annotation.sort_values(
            by=by, ascending=ascending
        ).index

        return self.subset(what="metabolites", ids=list(sorted_ids))

    def split(
        self,
        by: Literal["samples", "metabolites"] = "samples",
        columns: list[str] = [],
    ):
        if by == "samples":
            return self._split_by_sample_column(columns)
        elif by == "metabolites":
            return self._split_by_metabolite_column(columns)

    def _split_by_sample_column(self, sample_columns: list) -> dict:
        """

        Args:
            sample_columns:

        Returns:

        """
        split_datasets = {}
        for name, group in self.dataset.sample_metadata.groupby(by=sample_columns):
            temp_dataset = self.subset(what="samples", ids=list(group.index))
            split_datasets[name] = temp_dataset
        return split_datasets

    def _split_by_metabolite_column(self, metabolite_columns: list) -> dict:
        """

        Args:
            metabolite_columns:

        Returns:

        """
        split_dataset = {}
        for name, group in self.dataset.chemical_annotation.groupby(
            by=metabolite_columns
        ):
            temp_dataset = self.subset(what="metabolites", ids=list(group.index))
            split_dataset[name] = temp_dataset
        return split_dataset

    # TODO: implement dataset merging/concatenation
    """
    def concat(self, other):
        merged_data = pd.concat([self.data, other.data])
        merged_sample_metadata = pd.concat(
            [self.sample_metadata, other.sample_metadata]
        )
        merged_chemical_annotation = self.chemical_annotation.merge(
            other.chemical_annotation, left_index=True, right_index=True
        )
        return self.
    """
    """
    Utility functions
    """

    def merge_sample_metadata_data(self) -> pd.DataFrame:
        """

        Returns:

        """
        # Merge sample metadata and data by matching sample IDs
        merged = self.dataset.sample_metadata.merge(
            self.dataset.data, left_index=True, right_index=True, how="inner"
        )
        return merged

    def replace_metabolite_names_in_data(self, new_column: str):
        """

        Args:
            new_column:

        Raises:
            ValueError:
        """

        if new_column not in self.dataset.chemical_annotation.columns:
            raise ValueError(f"No column named {new_column} in the metabolite metadata")

        return self.dataset._setup(
            data=self.dataset.data,
            sample_metadata=self.dataset.sample_metadata,
            chemical_annotation=self.dataset.chemical_annotation,
            sample_id_column=self.dataset._sample_id_column,
            metabolite_id_column=new_column,
        )

    def replace_sample_names_in_data(self, new_index: str):
        # Check that the column exists in the sample metadata
        if new_index not in self.dataset.sample_metadata.columns:
            raise ValueError(f"No column named {new_index} in the sample metadata")
        new_data = self.dataset.data
        new_data.index = self.dataset.sample_metadata[new_index]
        return self.dataset._setup(
            data=self.dataset.data,
            sample_metadata=self.dataset.sample_metadata,
            chemical_annotation=self.dataset.chemical_annotation,
            sample_id_column=new_index,
            metabolite_id_column=self.dataset._metabolite_id_column,
        )



import pytest
import pandas as pd
from metabotk.parse_and_setup import (
    read_excel,
    read_tables,
    dataset_from_prefix,
    read_prefix,
    setup_data,
    setup_sample_metadata,
    setup_chemical_annotation,
)
import numpy as np


class Helper:
    @staticmethod
    def test_parsed_dict(parsed):
        assert isinstance(parsed, dict)
        assert list(parsed.keys()) == [
            "sample_metadata",
            "chemical_annotation",
            "data",
        ]
        assert parsed["sample_metadata"].shape == (46, 10)
        assert parsed["data"].shape == (46, 83)
        assert parsed["chemical_annotation"].shape == (82, 20)


@pytest.fixture
def helper():
    return Helper


def test_read_excel(helper):
    file_path = "tests/test_data/cdt_demo.xlsx"
    parsed = read_excel(file_path, data_sheet="Batch-normalized Data")
    helper.test_parsed_dict(parsed)


def test_read_tables(helper):
    data_path = "tests/test_data/data.csv"
    sample_metadata_path = "tests/test_data/sample_metadata.csv"
    chemical_annotation_path = "tests/test_data/chemical_annotation.csv"
    parsed = read_tables(sample_metadata_path, chemical_annotation_path, data_path)
    helper.test_parsed_dict(parsed)


def test_dataset_from_prefix():
    prefix = "tests/test_data/test"
    expected_dict = {
        "sample_metadata": "tests/test_data/test.samples",
        "chemical_annotation": "tests/test_data/test.metabolites",
        "data": "tests/test_data/test.data",
    }
    output = dataset_from_prefix(prefix)
    assert output == expected_dict


def test_read_prefix(helper):
    parsed = read_prefix("tests/test_data/test")
    helper.test_parsed_dict(parsed)


class TestSetupData:
    def test_setup_data_already_indexed(self):
        data = pd.read_csv("tests/test_data/data.csv")
        data_with_index = data.set_index("PARENT_SAMPLE_NAME")
        ready_data = setup_data(data_with_index, "PARENT_SAMPLE_NAME")
        assert ready_data.index.name == "PARENT_SAMPLE_NAME"

    def test_setup_data_not_indexed(self):
        data = pd.read_csv("tests/test_data/data.csv")
        ready_data = setup_data(data, "PARENT_SAMPLE_NAME")
        assert ready_data.index.name == "PARENT_SAMPLE_NAME"

    def test_setup_data_invalid_sample_id_column(self):
        data = pd.read_csv("tests/test_data/data.csv")
        with pytest.raises(ValueError):
            setup_data(data, "INVALID_COLUMN_NAME")

    def test_setup_data_column_types(self):
        data = pd.read_csv("tests/test_data/data.csv")
        data_with_index = data.set_index("PARENT_SAMPLE_NAME")
        ready_data = setup_data(data_with_index, "PARENT_SAMPLE_NAME")
        ready_data.columns = [int(i) for i in ready_data.columns]
        assert type(ready_data.columns[1]) is np.int64
        ready_data = setup_data(ready_data, "PARENT_SAMPLE_NAME")
        assert type(ready_data.columns[1]) is str


class TestSetupSampleMetadata:
    def test_setup_metadata_already_indexed(self):
        data = pd.read_csv("tests/test_data/sample_metadata.csv")
        data_with_index = data.set_index("PARENT_SAMPLE_NAME")
        ready_data = setup_sample_metadata(data_with_index, "PARENT_SAMPLE_NAME")
        assert ready_data.index.name == "PARENT_SAMPLE_NAME"

    def test_setup_metadata_not_indexed(self):
        data = pd.read_csv("tests/test_data/sample_metadata.csv")
        ready_data = setup_sample_metadata(data, "PARENT_SAMPLE_NAME")
        assert ready_data.index.name == "PARENT_SAMPLE_NAME"

    def test_setup_metadata_invalid_sample_id_column(self):
        data = pd.read_csv("tests/test_data/sample_metadata.csv")
        with pytest.raises(ValueError):
            setup_sample_metadata(data, "INVALID_COLUMN_NAME")

    def test_setup_metadata_column_types(self):
        data = pd.read_csv("tests/test_data/sample_metadata.csv")
        data_with_index = data.set_index("PARENT_SAMPLE_NAME")
        ready_data = setup_sample_metadata(data_with_index, "PARENT_SAMPLE_NAME")
        assert type(ready_data.columns[1]) is str

    def test_setup_metadata_duplicated_values(self):
        data = pd.read_csv("tests/test_data/sample_metadata.csv")
        with pytest.warns():
            setup_sample_metadata(data, "SUBGROUP")


class TestSetupChemicalAnnotation:
    def test_setup_metadata_already_indexed(self):
        data = pd.read_csv("tests/test_data/chemical_annotation.csv")
        data_with_index = data.set_index("CHEM_ID")
        ready_data = setup_chemical_annotation(data_with_index, "CHEM_ID")
        assert ready_data.index.name == "CHEM_ID"

    def test_setup_metadata_not_indexed(self):
        data = pd.read_csv("tests/test_data/chemical_annotation.csv")
        ready_data = setup_chemical_annotation(data, "CHEM_ID")
        assert ready_data.index.name == "CHEM_ID"

    def test_setup_metadata_invalid_sample_id_column(self):
        data = pd.read_csv("tests/test_data/chemical_annotation.csv")
        with pytest.raises(ValueError):
            setup_chemical_annotation(data, "INVALID_COLUMN_NAME")

    def test_setup_metadata_column_types(self):
        data = pd.read_csv("tests/test_data/chemical_annotation.csv")
        data_with_index = data.set_index("CHEM_ID")
        ready_data = setup_chemical_annotation(data_with_index, "CHEM_ID")
        assert type(ready_data.columns[1]) is str

    def test_setup_metadata_duplicated_values(self):
        data = pd.read_csv("tests/test_data/chemical_annotation.csv")
        with pytest.warns():
            setup_chemical_annotation(data, "SUPER_PATHWAY")



import pytest
import pandas as pd
from metabotk.providers_handler import MetabolonCDT
from metabotk.statistics_handler import StatisticsHandler
from metabotk.utils import parse_input
from metabotk.interface import MetaboTK


@pytest.fixture
def sample_metadata():
    # Path to sample metadata test data
    return pd.read_csv("data/sample_metadata.csv")


@pytest.fixture
def chemical_annotation():
    # Load chemical annotation test data
    return pd.read_csv("data/chemical_annotation.csv")


@pytest.fixture
def data():
    # Load main data test data
    return pd.read_csv("data/data.csv")


@pytest.fixture
def excel():
    # Load main data test data
    return "data/cdt_demo.xlsx"


@pytest.fixture
def metabotk_instance(data, sample_metadata, chemical_annotation):
    # Initialize MetaboTK instance for testing
    metabotk = MetaboTK()
    metabotk.import_tables(
        data=data,
        chemical_annotation=chemical_annotation,
        sample_metadata=sample_metadata,
    )
    return metabotk


class TestMetaboTK:

    def test_init(self):
        # Test MetaboTK initialization
        metabotk = MetaboTK()
        assert isinstance(metabotk.stats, StatisticsHandler)

    def test_setup_data(self, data, sample_metadata, chemical_annotation):
        # Test setup_data function
        metabotk = MetaboTK()
        metabotk._setup_data(
            data=data,
            chemical_annotation=chemical_annotation,
            sample_metadata=sample_metadata,
        )
        assert not metabotk.data.empty
        assert not metabotk.sample_metadata.empty
        assert not metabotk.chemical_annotation.empty

    def test_import_excel(self, data, sample_metadata, chemical_annotation):
        # Test import_excel function
        metabotk = MetaboTK()
        with pytest.raises(FileNotFoundError):
            metabotk.import_excel("nonexistent_file.xlsx", "data_sheet")

    def test_import_tables(self, data, sample_metadata, chemical_annotation):
        # Test import_tables function
        metabotk = MetaboTK()
        metabotk.import_tables(
            data=data,
            chemical_annotation=chemical_annotation,
            sample_metadata=sample_metadata,
        )
        assert not metabotk.data.empty
        assert not metabotk.sample_metadata.empty
        assert not metabotk.chemical_annotation.empty

    def test_merge_sample_metadata_data(self, metabotk_instance):
        # Test merge_sample_metadata_data function
        merged = metabotk_instance.merge_sample_metadata_data()
        assert not merged.empty

    def test_save_empty(self, metabotk_instance):
        # Test save_merged function
        metabotk_instance = MetaboTK()
        with pytest.raises(ValueError):
            metabotk_instance.save_merged("empty.tsv")

    def test_replace_column_names(self, metabotk_instance):
        # Test replace_column_names function
        with pytest.raises(ValueError):
            metabotk_instance.replace_column_names("nonexistent_column")

    def test_drop_missing(self, metabotk_instance):
        # Test drop_missing function
        dropped_columns = (
            metabotk_instance.stats.missing_handler.drop_missing_from_dataframe(
                metabotk_instance.data, axis=0
            )
        )
        assert isinstance(dropped_columns, pd.DataFrame)

    def test_split_by_sample_column(self, metabotk_instance):
        # Test split_by_sample_column function
        split_data = metabotk_instance.split_by_sample_column("SUPERGROUP")
        assert isinstance(split_data, dict)

    def test_total_sum_abundance(self, metabotk_instance):
        # Test total_sum_abundance function
        tsa = metabotk_instance.stats.total_sum_abundance(metabotk_instance.data)
        assert isinstance(tsa, pd.Series)

    def test_compute_stats(self, metabotk_instance):
        # Test compute_stats function
        sample_stats = metabotk_instance.stats.sample_stats()
        metabolite_stats = metabotk_instance.stats.metabolite_stats()
        assert not sample_stats.empty
        assert not metabolite_stats.empty

    def test_get_pca(self, metabotk_instance):
        # Test get_pca function
        pca = metabotk_instance.dimensionality_reduction.get_pca(
            n_components=3,
            get_pca_object=False,
        )
        print(pca.shape)
        assert isinstance(pca, pd.DataFrame)
        assert pca.shape == (
            metabotk_instance.data.shape[0],
            3 + len(metabotk_instance.sample_metadata.columns),
        )

    # Add more tests for other methods of MetaboTK class
    # ...


@pytest.mark.parametrize("data_sheet", ["data", "other_sheet"])
def test_import_excel(data_sheet):
    # Test import_excel function
    metabotk = MetaboTK()
    with pytest.raises(FileNotFoundError):
        metabotk.import_excel("nonexistent_file.xlsx", data_sheet)


class TestMetaboTKReplaceColumnNames:

    def test_replace_column_names_nonexistent_column(self, metabotk_instance):
        # Test replace_column_names function when the specified column does not exist
        with pytest.raises(ValueError):
            metabotk_instance.replace_column_names("nonexistent_column")


class TestMetaboTKOutliers:

    def test_remove_outliers(self, metabotk_instance):
        # Test remove_outliers function
        with_outliers = metabotk_instance.data.copy()
        removed_outliers = metabotk_instance.stats.outlier_handler.remove_outliers(
            with_outliers, threshold=5
        )
        # Check if any NaN values present, indicating removal of outliers
        assert not with_outliers.equals(removed_outliers)


class TestMetaboTKExtraction:

    @pytest.mark.parametrize("metabolites_to_extract", ["50", "171"])
    def test_extract_metabolites(self, metabotk_instance, metabolites_to_extract):
        # Test extract_metabolites function
        extracted_data = metabotk_instance.extract_metabolites(metabolites_to_extract)
        assert not extracted_data.empty

    @pytest.mark.parametrize(
        "samples_to_extract", ["INTR-03192 [COPY 2]", "INTR-03231 [COPY 2]"]
    )
    def test_extract_samples(self, metabotk_instance, samples_to_extract):
        # Test extract_samples function
        extracted_data = metabotk_instance.extract_samples(samples_to_extract)
        assert not extracted_data.empty

    @pytest.mark.parametrize("metabolites_to_extract", ["50", "171"])
    def test_extract_chemical_annotations(
        self, metabotk_instance, metabolites_to_extract
    ):
        # Test extract_chemical_annotations function
        extracted_annotations = metabotk_instance.extract_chemical_annotations(
            metabolites_to_extract
        )
        assert not extracted_annotations.empty


class TestMetaboTKSplit:

    @pytest.mark.parametrize("column", ["SUPERGROUP", "SUBGROUP"])
    def test_split_by_sample_column(self, metabotk_instance, column):
        # Test split_by_sample_column function
        split_data = metabotk_instance.split_by_sample_column(column)
        assert isinstance(split_data, dict)

    @pytest.mark.parametrize("column", ["SUPER_PATHWAY", "SUB_PATHWAY"])
    def test_split_by_metabolite_column(self, metabotk_instance, column):
        # Test split_by_metabolite_column function
        split_data = metabotk_instance.split_by_metabolite_column(column)
        assert isinstance(split_data, dict)


# Add more test classes for other classes if necessary

if __name__ == "__main__":
    pytest.main()


def test_merge_data_sample_metadata():
    test_instance = MetaboTK(sample_id_column="PARENT_SAMPLE_NAME")
    test_instance.import_excel("data/cdt_demo.xlsx", data_sheet="batch_normalized_data")
    merged_data = test_instance.merge_sample_metadata_data()
    # Add assertions here to check if the merged data is as expected
    assert merged_data.shape[0] == 46  # Assuming 46 samples in the sample metadata
    assert merged_data.shape[1] == 91  # Assuming 91 columns in the metabolomic data
    assert (
        merged_data.index.name == "PARENT_SAMPLE_NAME"
    )  # Assuming 5 columns in the metabolomic data


def test_sample_id_column_warning():
    with pytest.raises(ValueError):
        # Initialize MetaboTK with a data file that doesn't contain the specified sample_id_column
        metabotk_instance = MetaboTK(sample_id_column="CLIENT_IDENTIFIER")
        metabotk_instance.import_excel(
            "data/cdt_demo.xlsx", data_sheet="batch_normalized_data"
        )


def test_unsupported_data_provider():
    with pytest.raises(NotImplementedError):
        # Initialize MetaboTK with a data provider not supported
        metabotk_instance = MetaboTK(data_provider="biocrates")



import warnings
import pytest
import numpy as np
import pandas as pd
import metabotk.missing_handler as missing

# setup test data
test_data = pd.read_csv("tests/test_data/data.csv")
test_data = test_data[test_data.columns[5:10]].iloc[10:20]
test_values_series = test_data["212"]
test_values_list = test_values_series.tolist()


class TestValidateThreshold:
    def test_string_threshold(self):
        with pytest.raises(TypeError):
            missing._validate_threshold("A")
        with pytest.raises(ValueError):
            missing._validate_threshold(12)


class TestDetectMissingValues:
    def test_input_list(self):
        data = test_values_list
        assert missing._detect_missing(data).sum() == 2

    def test_input_series(self):
        data = test_values_series
        assert missing._detect_missing(data).sum() == 2

    def test_no_missing(self):
        data = np.array([1, 2, 3, 4, 5])
        assert not any(missing._detect_missing(data))

    def test_single_missing(self):
        data = np.array([1, 2, 3, np.nan, 5])
        assert np.array_equal(
            missing._detect_missing(data), [False, False, False, True, False]
        )

    def test_multiple_missing(self):
        data = np.array([1, np.nan, 1, np.nan, 1, 2, 3])
        assert np.array_equal(
            missing._detect_missing(data),
            [False, True, False, True, False, False, False],
        )

    def test_empty_input(self):
        with warnings.catch_warnings():
            # Filter out the specific warning related to nanmean on an empty slice
            warnings.filterwarnings("ignore", message="Mean of empty slice")
            data = np.array([])
            detected_missing = missing._detect_missing(data)
        assert not any(detected_missing)


# count_missing_values
class TestCountMissingValues:
    data = test_data

    def test_count_missing_values_column_wise(self):
        missing_counts = missing.count_missing_in_dataframe(self.data, axis=0)
        missing_counts_to_assert = pd.Series([2, 0, 0, 0, 4])
        missing_counts_to_assert.index = self.data.columns
        assert missing_counts_to_assert.equals(missing_counts)

    def test_count_missing_values_row_wise(self):
        missing_counts = missing.count_missing_in_dataframe(self.data, axis=1)
        missing_counts_to_assert = pd.Series([0, 1, 0, 1, 2, 0, 0, 0, 1, 1])
        missing_counts_to_assert.index = self.data.index
        assert missing_counts_to_assert.equals(missing_counts)


# drop_columns_with_missing_over_threshold
class TestDropOverThreshold:
    data = test_data

    def test_drop_columns_with_missing_over_threshold_0(self):
        dropped = missing._drop_columns_with_missing(self.data, threshold=0)
        assert np.array_equal(
            list(dropped.columns),
            ["212", "273"],
        )

    def test_drop_rows_with_missing_over_threshold_0(self):
        dropped = missing._drop_rows_with_missing(self.data, threshold=0)
        assert np.array_equal(list(dropped.index), [11, 13, 14, 18, 19])

    def test_drop_columns_with_missing_over_threshold_1(self):
        dropped = missing._drop_columns_with_missing(self.data, threshold=1)
        assert len(dropped.columns) == 0

    def test_drop_rows_with_missing_over_threshold_1(self):
        dropped = missing._drop_rows_with_missing(self.data, threshold=1)
        assert len(dropped.index) == 0

    def test_drop_columns_with_missing_over_threshold_three(self):
        dropped = missing._drop_columns_with_missing(self.data, threshold=0.3)
        assert np.array_equal(list(dropped.columns), ["273"])

    def test_drop_rows_with_missing_over_threshold_three(self):
        dropped = missing._drop_rows_with_missing(self.data, threshold=0.3)
        assert np.array_equal(list(dropped.index), [14])


class TestDropOverThreshold2:
    data = test_data

    def test_drop_columns_with_missing_over_threshold_0(self):
        remaining = missing.drop_missing_from_dataframe(self.data, axis=0, threshold=0)
        assert np.array_equal(
            list(remaining.columns),
            ["229", "250", "254"],
        )

    def test_drop_rows_with_missing_over_threshold_0(self):
        remaining = missing.drop_missing_from_dataframe(self.data, axis=1, threshold=0)
        assert np.array_equal(
            list(remaining.index),
            [10, 12, 15, 16, 17],
        )

    def test_drop_columns_with_missing_over_threshold_1(self):
        remaining = missing.drop_missing_from_dataframe(self.data, axis=0, threshold=1)
        assert self.data.equals(remaining)

    def test_drop_rows_with_missing_over_threshold_1(self):
        remaining = missing.drop_missing_from_dataframe(self.data, axis=1, threshold=1)
        assert self.data.equals(remaining)



from .metabolomic_dataset import MetabolomicDataset
from .dataset_io import DatasetIO
from .dataset_operations import DatasetOperations
from .statistics_handler import Statistics

from .main import MetaboTK

__all__ = ["MetaboTK"]

# from .main import MetaboTK

"""
from .outliers_handler import OutlierHandler
from .missing_handler import MissingDataHandler
from .statistics_handler import StatisticsHandler
from .dataset_manager import DatasetManager
from .models_handler import ModelsHandler
from .visualization_handler import Visualization
from .dimensionality_reduction import DimensionalityReduction
from .imputation import ImputationHandler
from .feature_selection import FeatureSelection
from .normalization import NormalizationHandler
from .scaling import ScalingHandler

from ._version import __version__
from .interface import MetaboTK

__all__ = [
    "__version__",
]
"""
